
# q-expansion of newform 9680.2.a.bs, downloaded from the LMFDB on 14 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 9680
weight = 2
poly_data = [-4, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [0, 1], [1, 0], [0, -1], [0, 0], [-2, 0], [-2, 1], [0, 1], [0, -2], [-2, -3], [-4, -1], [2, -3], [-2, 0], [0, 4], [8, 2], [2, 1], [4, -2], [-10, 3], [4, -4], [4, -3], [2, 0], [0, -6], [0, 2], [-10, -1], [2, 2], [-6, 4], [0, -4], [-16, 0], [2, -8], [-14, -2], [4, 4], [0, -3], [-6, 10], [12, 0], [-10, 3], [8, 2], [2, 5], [-16, 5], [-8, -1], [6, -4], [-4, 4], [6, -4], [0, -4], [-18, 5], [-10, 6], [-12, 5], [16, -3], [-8, -2], [8, -2], [6, -12], [-2, -7], [0, 10], [6, 4], [4, 10], [2, -12], [-8, 5], [6, -10], [8, 8], [-2, -2], [-2, 4], [-8, -6], [6, -8], [0, -10], [28, -1], [10, 8], [26, -5], [-12, 8], [-10, 3], [-24, 2], [10, -8], [-14, -8], [16, 4], [-16, 12], [-2, 0], [20, 2], [16, 6], [22, 4], [-18, 0], [-10, 1], [-2, -2], [-4, 12], [-18, 10], [-16, 0], [18, 6], [-24, -4], [4, 0], [18, 0], [-26, -1], [-18, -1], [-8, 6], [8, -7], [8, -16], [-8, 4], [-16, 9], [4, -8], [-4, -4], [6, 6], [-22, 12], [-24, 8], [-10, 17], [-16, 0], [-26, -8], [-8, 2], [-18, 18], [-8, 15], [-6, 12], [-40, 3], [-22, 8], [20, -1], [30, 0], [8, 5], [14, -12], [-22, 6], [-12, 8], [4, 5], [-10, -11], [-16, -3], [8, 2], [2, -5], [32, -5], [6, 4], [-18, 21], [-2, -10], [8, 7], [12, -6], [-42, 3], [22, -8], [-36, 11], [8, -10], [14, 2], [-12, -12], [0, -1], [-12, 11], [-26, 0], [6, 12], [6, -12], [-30, -3], [-24, -12], [-34, 4], [14, -8], [40, -5], [2, 0], [32, 8], [16, -8], [-10, 6], [-32, 8], [14, -4], [6, -15], [44, -2], [16, -10], [22, 4], [2, 0], [-24, 7], [-20, -4], [-32, 9], [20, -7], [8, 0], [22, 9], [42, 0], [22, -7], [32, 3], [6, 11], [-32, 17], [4, -4], [-30, 2], [-24, 2], [-32, 0], [22, -14], [38, 8], [-46, 5], [0, 9], [-34, 8], [40, -10], [-34, -3], [16, -8], [30, -15], [-20, 8], [30, -2], [-32, 5], [-2, -1], [-32, 2], [40, -3], [18, -9], [14, 5], [8, -2], [22, -4], [14, 14], [36, 0], [-10, 12], [-16, 0], [2, -2], [32, -20], [-36, -6], [22, -26], [32, -4], [-22, -8], [-34, 2], [38, 12], [-14, 20], [-24, 13], [-2, 5], [-8, -10], [-6, -1], [-2, -14], [4, -16], [-14, 5], [-8, -8], [16, -8], [-2, 0], [20, -20], [14, -25], [-2, 22], [-16, 20], [-8, 11], [32, -8], [30, 1], [-32, 13], [-18, -20], [16, 0], [-14, -9], [-14, 0], [-40, 2], [-18, -13], [-8, -12], [-56, 8], [6, 27], [-14, 32], [44, 9], [-4, -4], [68, 0], [46, -12], [32, -5], [56, -2], [14, -4], [-20, 24], [16, -11], [22, -31], [6, 4], [12, 0], [-12, 3], [40, 11], [4, 8], [0, 4], [14, -16], [18, -12], [40, -10], [-64, -4], [12, -30], [16, 5], [32, -27], [14, -28], [70, -2], [40, -6], [-34, 9], [30, 10], [24, -3], [-18, 6], [0, 18], [38, 8], [-42, 3], [-52, 4], [-32, 12], [62, 7], [-18, 12], [-38, 10], [4, -28], [6, 2], [-58, 7], [-16, -16], [6, 8], [14, 16], [-16, -1], [-6, -10], [-8, -26], [-26, 17], [-16, -14], [-36, 12], [-38, -4], [30, -28], [-24, 15], [-40, 15], [-40, -4], [4, 12], [6, 19], [-8, 4], [28, 15], [-6, 4], [-26, -4], [-68, 9], [46, -8], [6, 22], [12, -24], [46, -3], [32, -13], [-18, 24], [-14, -20], [8, 16], [10, -9], [44, 0], [32, 6], [-42, -1], [-34, 30], [0, -12], [20, 8], [12, -6], [-22, -26], [12, 24], [-10, 26], [-44, 19], [-26, -6], [52, 8], [-26, -12], [6, -10], [24, 5], [24, -1], [-2, -12], [-28, -2], [-24, 8], [10, 12], [70, -2], [8, 19], [82, -4], [-38, 12], [-8, -2], [-10, -13], [22, 1], [-28, -10], [4, -24], [-24, 3], [-10, -14], [14, -39], [14, 28], [56, -8], [40, -6], [56, 1], [8, -31], [-50, 12], [6, 17], [-10, 11], [-8, -21], [70, -4], [26, 8], [14, 17], [12, -27], [10, 9], [-8, -33], [-18, -2], [0, -21], [16, 28], [26, -29], [8, -27], [58, 12], [-42, 30], [-40, -3], [-66, 11], [-18, -21], [52, -21], [16, -5], [18, 16], [0, -20], [-42, 12], [-2, 26], [16, 20], [-16, 3], [64, -19], [-54, -10], [22, 0], [16, -21], [6, 18], [36, -12], [-76, 8], [-20, 4], [22, -5], [-64, 14], [-66, 0], [44, -12], [32, -22], [38, 1], [-26, 26], [-26, -17], [-26, -6], [-22, -12], [-68, -8], [-26, -5], [12, -12], [-32, 16], [-40, -24], [-62, 7], [-56, -12], [48, -6], [-18, -13], [42, -21], [28, -4], [36, -4], [16, 16], [-6, -4], [-24, -14], [-18, -11], [4, 18], [-18, 27], [-26, 25], [-54, 0], [-4, 0], [-46, 22], [58, 8], [8, -22], [-22, -21], [-34, -4], [0, -25], [-20, -6], [18, -28], [-18, -6], [-12, 0], [20, -32], [-46, -8], [-10, 42], [0, -4], [-16, 16], [2, -20], [72, -13], [-42, -18], [62, -22], [24, -12], [-40, 25], [-22, -4], [-14, -13], [40, -21], [14, 18], [-52, 32], [24, -26], [38, -21], [-24, -11], [20, 22], [40, -38], [-22, 11], [50, -20], [-50, 4], [6, 20], [-4, -8], [8, 40], [-32, 3], [22, -15], [54, -13], [-48, -12], [86, 2], [-42, -7], [-64, 2], [8, -43], [-2, -29]]
